//Apron Tutorials

#include "CCamera.hpp"


/////////////////////////////////////////////////////////////////////////////////////////////////
//										THE CCAMERA POSITION CAMERA
/////////////////////////////////////////////////////////////////////////////////////////////////
void CCamera::Position_Camera(float pos_x,  float pos_y,  float pos_z,
							  float view_x, float view_y, float view_z,
							  float up_x,   float up_y,   float up_z)
{
	mPos	= D3DXVECTOR3(pos_x,  pos_y,  pos_z ); // set position
	mView	= D3DXVECTOR3(view_x, view_y, view_z); // set view
	mUp		= D3DXVECTOR3(up_x,   up_y,   up_z  ); // set the up vector	
}


/////////////////////////////////////////////////////////////////////////////////////////////////
//										THE CCAMERA MOVE CAMERA
/////////////////////////////////////////////////////////////////////////////////////////////////
void CCamera::Move_Camera(float speed)
{
	D3DXVECTOR3 vVector = mView - mPos;	// Get the view vector
	
	// forward positive camera speed and backward negative camera speed.
	mPos.x  = mPos.x  + vVector.x * speed;
	mPos.z  = mPos.z  + vVector.z * speed;
	mView.x = mView.x + vVector.x * speed;
	mView.z = mView.z + vVector.z * speed;
}

/************************
*  void CCamera::Strafe_Camera(float speed)
*	Args: float speed
*	Return: void
*
*  Move the camera perpendicularly to the view vector
*
*  by Aaron MacDonald
*  13 May 2010 
************************/
void CCamera::Strafe_Camera(float speed)
{
	D3DXVECTOR3 vVector = mView - mPos;	// Get the view vector
	D3DXMATRIX mRotation;

	D3DXMatrixRotationY(&mRotation, 1.57);

	D3DXVec3TransformCoord(&vVector, &vVector, &mRotation);

	// forward positive camera speed and backward negative camera speed.
	mPos.x  = mPos.x  + vVector.x * speed;
	mPos.z  = mPos.z  + vVector.z * speed;
	mView.x = mView.x + vVector.x * speed;
	mView.z = mView.z + vVector.z * speed;
}

/************************
*  void CCamera::Fly_Camera(float speed)
*	Args: float speed
*	Return: void
*
*  Move the camera up and down the y axis
*
*  by Aaron MacDonald
*  13 May 2010 
************************/
void CCamera::Fly_Camera(float speed)
{
	D3DXVECTOR3 vVector(0, 500, 0);	// Get the view vector

	// forward positive camera speed and backward negative camera speed.
	mPos += vVector * speed;
	mView += vVector * speed;
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//										THE CCAMERA ROTATE VIEW
/////////////////////////////////////////////////////////////////////////////////////////////////
void CCamera::Rotate_View(float speed)
{
	D3DXVECTOR3 vVector = mView - mPos;	// Get the view vector

	mView.z = (float)(mPos.z + sin(speed)*vVector.x + cos(speed)*vVector.z);
	mView.x = (float)(mPos.x + cos(speed)*vVector.x - sin(speed)*vVector.z);
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//										THE CCAMERA MOUSE MOVE
/////////////////////////////////////////////////////////////////////////////////////////////////
void CCamera::Mouse_Move(int wndWidth, int wndHeight)
{
	POINT mousePos;	
	int mid_x = wndWidth  >> 1;	
	int mid_y = wndHeight >> 1;	
	float angle_y  = 0.0f;				
	float angle_z  = 0.0f;							
	
	GetCursorPos(&mousePos);	// Get the 2D mouse cursor (x,y) position					
	
	if( (mousePos.x == mid_x) && (mousePos.y == mid_y) ) return;

	SetCursorPos(mid_x, mid_y);	// Set the mouse cursor in the center of the window						

	// Get the direction from the mouse cursor, set a resonable maneuvering speed
	angle_y = (float)( (mid_x - mousePos.x) ) / 1050;		
	angle_z = (float)( (mid_y - mousePos.y) ) / 1050;

	// The higher the value is the faster the camera looks around.
	mView.y += angle_z*1050;
	
	// limit the rotation around the x-axis
	//if((mView.y - mPos.y) > 8)  mView.y = mPos.y + 8;
	//if((mView.y - mPos.y) <-8)  mView.y = mPos.y - 8;
	
	Rotate_View(angle_y); // Rotate
}



//Ronny Andr Reierstad
//www.morrowland.com
//apron@morrowland.com