#include "Entity.hpp"
#include "GraphicsManager.hpp"
#include "EntityManager.hpp"

DWORD Entity::numMaterials = NULL;
LPDIRECT3DTEXTURE9 Entity::glow = NULL;
GraphicsManager *Entity::gMan = NULL;



void Entity::InitializeClass()
{

}

Entity::Entity(D3DXVECTOR3 _position, LPCTSTR mesh_name, LPCTSTR texture_name)
{
	/* setup variables */
	target = NULL;
	gMan = GraphicsManager::Instance();
	position = _position;
	hit_points = 15;
	dead = false;

		/* setup mesh */
	LPD3DXBUFFER bufMeshMaterial;
	D3DXLoadMeshFromX(mesh_name, 
		D3DXMESH_SYSTEMMEM,
		gMan->d3ddev,
		NULL,
		&bufMeshMaterial,
		NULL,
		&numMaterials,
		&mesh);
	vector<DWORD> adjacencyBuffer(mesh->GetNumFaces() * 3);
	mesh->GenerateAdjacency(0.0f, &adjacencyBuffer[0]);
	mesh->OptimizeInplace(D3DXMESHOPT_ATTRSORT |
		D3DXMESHOPT_COMPACT |
		D3DXMESHOPT_VERTEXCACHE,&adjacencyBuffer[0],
		0, 0, 0);


 // end Setup()

	/* setup texture and glow map */
	D3DXCreateTextureFromFile(gMan->d3ddev,    // the Direct3D device
                              texture_name,    // the filename of the texture
                              &texture);			// the address of the texture storage
	D3DXCreateTextureFromFile(gMan->d3ddev,    // the Direct3D device
                              "glow.png",    // the filename of the texture
                              &glow);			// the address of the texture storage



	currentDestination.movementInProgress = false;

	
}

void Entity::Draw()
{

	D3DXMATRIX matTranslate;

	GraphicsManager * gMan = GraphicsManager::Instance();
	D3DXMatrixTranslation(&matTranslate, position.x, position.y, position.z);
	gMan->d3ddev->SetTransform(D3DTS_WORLD, &(matRotation*matTranslate));



	
	gMan->d3ddev->SetTexture(0, texture);
	mesh->DrawSubset(0);
	
	gMan->d3ddev->SetTexture(0, NULL);

		
}

void Entity::Logic()
{

}

void Entity::Attack(Entity * _target)
{
}


btRigidBody* Entity::GetRigidBody()
{
	return fallRigidBody;
}

void Entity::SetPosition(D3DXVECTOR3 _position)
{
	position = _position;
}

void Entity::AddLinearVelocity(const btVector3 & v)
{
	linearVelocity = linearVelocity + v;
}

//http://forums.xna.com/forums/t/10228.aspx
// face a vector position
void Entity::Face(D3DXVECTOR3 vec)
{
	D3DXVECTOR3 U(0, 1, 0);
	D3DXVECTOR3 D = (position - vec);
	D3DXVECTOR3 Right;
	D3DXVec3Cross(&Right, &U, &D);
	D3DXVec3Normalize(&Right, &Right);
	D3DXVECTOR3 Backwards;
	D3DXVec3Cross(&Backwards, &Right, &U);
	D3DXVec3Normalize(&Backwards, &Backwards);
	D3DXVECTOR3 Up;
	D3DXVec3Cross(&Up, &Backwards, &Right);
	D3DXMATRIX temp(Right.x, Right.y, Right.z, 0, Up.x, Up.y, Up.z, 0, Backwards.x, Backwards.y, Backwards.z, 0, 0, 0, 0, 1);
	matRotation = temp;
	D3DXVec3Normalize(&D, &D);
	btTransform trans;
	trans.setOrigin(fallRigidBody->getWorldTransform().getOrigin());
	D3DXQUATERNION rot;
	D3DXQuaternionRotationMatrix(&rot, &matRotation);
	trans.setRotation(btQuaternion(rot.x, rot.y, rot.z, rot.w));
	fallRigidBody->setWorldTransform(trans);
}

void Entity::MoveTo(D3DXVECTOR3 vec)
{
	currentDestination.destination = vec;
	currentDestination.movementInProgress = true;
	Face(vec);
}

Entity::~Entity()
{

}

Particle::Particle(D3DXVECTOR3 _position, D3DXVECTOR3 _direction)
{
	position = _position;
	direction = _direction;
	life_time_in_seconds = 500;
	time_at_last_update_seconds = 0;
	time_at_creation_seconds = timer.GetTime();
	
}

void Particle::Update()
{
	time_at_last_update_seconds = timer.GetTime();
	position = position + direction;

}

bool Particle::ShouldDie()
{
	if(time_at_last_update_seconds - time_at_creation_seconds >= life_time_in_seconds)
		return true;
	return false;
}


void Particle::Draw()
{
	GraphicsManager::DrawPoint(position);
	
}

ParticleSystem::ParticleSystem(D3DXVECTOR3 * _position, D3DXVECTOR3 * _direction, D3DXVECTOR3 _offset)
{
	position = _position;
	direction = _direction;
	offset = _offset;
	time_at_last_update_seconds = 0;
	timer.Initialize();
	time_at_last_new_particle = 0;
	srand ( time(NULL) ); 
		
	// setup point sprite
	D3DXCreateTextureFromFileExA(GraphicsManager::Instance()->d3ddev, //device
                             "flare.jpg",        //file name
                             D3DX_DEFAULT,      //width
                             D3DX_DEFAULT,      //height
                             D3DX_DEFAULT,      //mip levels
                             NULL,              //usage
                             D3DFMT_UNKNOWN,    //texture color format
                             D3DPOOL_MANAGED,   //memory class
                             D3DX_DEFAULT,      //filter
                             D3DX_DEFAULT,      //mip filter
                             0,        //color key
                             NULL,              //source info
                             NULL,              //pallette
                             &texture);    //texture object
}

void ParticleSystem::Update()
{
	float current_seconds = timer.GetTime();
	list<Particle> updatedParticles;
	if(current_seconds - time_at_last_update_seconds >= 10)
	{
		list<Particle>::iterator it;
		// clear out dead particles
		for ( it=particles.begin() ; it != particles.end(); it++ )
		{
			(*it).Update();
			if((*it).ShouldDie() == false)
				updatedParticles.push_back((*it));
		}
		time_at_last_update_seconds = timer.GetTime();
	}
	particles = updatedParticles;
	// add 3 particles every 1 ms (hmm that doesn't make sense)
	if(current_seconds - time_at_last_new_particle >= 1)
	{
		D3DXVECTOR3 tempdir = *direction;
		particles.push_back(Particle( *position+offset+(-tempdir*30), D3DXVECTOR3(-tempdir.x * (rand() %5), -tempdir.y * (rand() %5), -tempdir.z * (rand() %5) )));
		particles.push_back(Particle( *position+offset+(-tempdir*30), D3DXVECTOR3(-tempdir.x * (rand() %5), -tempdir.y * (rand() %5), -tempdir.z * (rand() %5) )));
		particles.push_back(Particle( *position+offset+(-tempdir*30), D3DXVECTOR3(-tempdir.x * (rand() %5), -tempdir.y * (rand() %5), -tempdir.z * (rand() %5) )));
		time_at_last_new_particle = timer.GetTime();
	}
}

void ParticleSystem::Draw()
{
	GraphicsManager::Instance()->d3ddev->SetTexture(0, texture);
	list<Particle>::iterator it;
	for ( it=particles.begin() ; it != particles.end(); it++ )
	{
		(*it).Draw();
	}
	GraphicsManager::Instance()->d3ddev->SetTexture(0, NULL);
}

float EntityDestination::DistanceFromDestination(D3DXVECTOR3 currentPosition)
{
	return D3DXVec3Length(&(destination - currentPosition));
}

void Entity::Update()
{
}

MousePlayer::MousePlayer(D3DXVECTOR3 _position, LPCTSTR mesh_name, LPCTSTR texture_name) : Entity(_position, mesh_name, texture_name)
{
	LPD3DXBUFFER bufMeshMaterial;
	player_moving_right = false;
	player_moving_left = false;
	frame = 0;
	D3DXLoadMeshFromX("Vernin_Right_Step_02.X", 
		D3DXMESH_SYSTEMMEM,
		gMan->d3ddev,
		NULL,
		&bufMeshMaterial,
		NULL,
		&numMaterials,
		&vernin_right);
	D3DXLoadMeshFromX("Vernin_Standing_02.X", 
		D3DXMESH_SYSTEMMEM,
		gMan->d3ddev,
		NULL,
		&bufMeshMaterial,
		NULL,
		&numMaterials,
		&vernin_stand);
	D3DXLoadMeshFromX("Vernin_Left_Step_02.X", 
		D3DXMESH_SYSTEMMEM,
		gMan->d3ddev,
		NULL,
		&bufMeshMaterial,
		NULL,
		&numMaterials,
		&vernin_left);
	D3DXLoadMeshFromX("Vernin_Flying_02.X", 
		D3DXMESH_SYSTEMMEM,
		gMan->d3ddev,
		NULL,
		&bufMeshMaterial,
		NULL,
		&numMaterials,
		&vernin_jump);
	

	btConvexHullShape* fallShape = new btConvexHullShape();
	LPDIRECT3DVERTEXBUFFER9 v_buffer;
	mesh->GetVertexBuffer(&v_buffer);
	CUSTOMVERTEX * thisVertex;
	hit_points = 15;
	v_buffer->Lock(0,0,(void**)&thisVertex ,0);
	// make collision shape from mesh vertices
	for(int i = 0; i < mesh->GetNumVertices(); i++)
	{
		fallShape->addPoint(btVector3(thisVertex->X, thisVertex->Y, thisVertex->Z));
		thisVertex++;
	}
	v_buffer->Unlock();
	v_buffer->Release();
	btDefaultMotionState* fallMotionState = new btDefaultMotionState(btTransform(btQuaternion(0,0,0,1),btVector3(position.x,position.y,position.z)));
	btScalar mass = 1;
	btVector3 fallInertia(0,-1,0);
	fallShape->calculateLocalInertia(mass,fallInertia);
	btRigidBody::btRigidBodyConstructionInfo fallRigidBodyCI(mass,fallMotionState,fallShape,fallInertia);
	fallRigidBody = new btRigidBody(fallRigidBodyCI);
	//fallRigidBody->setCollisionFlags(btCollisionObject::CF_NO_CONTACT_RESPONSE);
	//fallRigidBody->setCollisionFlags(fallRigidBody->getCollisionFlags() | btCollisionObject::CF_NO_CONTACT_RESPONSE);
	// setup handle to this object (from the collision manifold)
	pCon = new PhysicsContainer();
	pCon->entity = this;
	pCon->type = 1;
	pCon->dead = false;
	fallRigidBody->setUserPointer(pCon);
	fallRigidBody->setSleepingThresholds (0.0, 0.0);
	fallRigidBody->setAngularFactor (0.0);
	PhysicsManager::Instance()->GetDynamicsWorld()->addRigidBody(fallRigidBody);
	D3DXMATRIX identity; 
	D3DXMatrixIdentity( &identity ); 
	weapon = NULL;
	matRotation = identity;
	linearVelocity = btVector3(0,0,0);
	animationTimer.Initialize();
	time_since_last_frame = 0;
}

void MousePlayer::Draw()
{
	D3DXMATRIX matTranslate;
	GraphicsManager * gMan = GraphicsManager::Instance();
	D3DXMatrixTranslation(&matTranslate, position.x, position.y, position.z);
	gMan->d3ddev->SetTransform(D3DTS_WORLD, &(matRotation*matTranslate));
	gMan->d3ddev->SetTexture(0, texture);
	animationTimer.Update();
	time_since_last_frame += animationTimer.Get_dt();
	switch(frame)
	{
	case 0:
		for(int i = 0; i < numMaterials; i++)
		{
			vernin_right->DrawSubset(i);
		}
		break;
	case 1:
		for(int i = 0; i < numMaterials; i++)
		{
			vernin_stand->DrawSubset(i);
		}
		break;
	case 2:
		for(int i = 0; i < numMaterials; i++)
		{
			vernin_left->DrawSubset(i);
		}
		break;
	case 3:
		for(int i = 0; i < numMaterials; i++)
		{
			vernin_stand->DrawSubset(i);
		}
		break;
	}
	if(time_since_last_frame > 30.0f)
	{
		frame++;
		time_since_last_frame = 0;
	}
	if(frame > 3)
		frame = 0;
	
	gMan->d3ddev->SetTexture(0, NULL);

}

void MousePlayer::Update()
{
	if(GraphicsManager::Instance()->selectedEntity != NULL)
	{
		if(GraphicsManager::Instance()->selectedEntity->pCon->type == 2 && abs(GraphicsManager::Instance()->selectedEntity->position.x - position.x) <= 100)
		{
			
			weapon = GraphicsManager::Instance()->selectedEntity;
			GraphicsManager::Instance()->selectedEntity = NULL;
		}
	}
	if(weapon != NULL)
	{
		weapon->position = position * 500;
	}
}

StaticEntity::StaticEntity(D3DXVECTOR3 _position, LPCTSTR mesh_name, LPCTSTR texture_name) : Entity(_position, mesh_name, texture_name)
{
	/* setup physics */
	btConvexHullShape* fallShape = new btConvexHullShape();
	LPDIRECT3DVERTEXBUFFER9 v_buffer;
	mesh->GetVertexBuffer(&v_buffer);
	CUSTOMVERTEX * thisVertex;
	v_buffer->Lock(0,0,(void**)&thisVertex ,0);
	// make collision shape from mesh vertices
	for(int i = 0; i < mesh->GetNumVertices(); i++)
	{
		fallShape->addPoint(btVector3(thisVertex->X, thisVertex->Y, thisVertex->Z));
		thisVertex++;
	}
	v_buffer->Unlock();
	v_buffer->Release();
	btDefaultMotionState* fallMotionState = new btDefaultMotionState(btTransform(btQuaternion(0,0,0,1),btVector3(position.x,position.y,position.z)));
	btScalar mass = 1;
	btVector3 fallInertia(0,0,0);
	fallShape->calculateLocalInertia(mass,fallInertia);
	btRigidBody::btRigidBodyConstructionInfo fallRigidBodyCI(0,fallMotionState,fallShape,btVector3(0,0,0));
	fallRigidBody = new btRigidBody(fallRigidBodyCI);
	fallRigidBody->setFriction(0.1f);
	//fallRigidBody->setCollisionFlags(btCollisionObject::CF_NO_CONTACT_RESPONSE);
	//fallRigidBody->setCollisionFlags(fallRigidBody->getCollisionFlags() | btCollisionObject::CF_NO_CONTACT_RESPONSE);
	// setup handle to this object (from the collision manifold)
	pCon = new PhysicsContainer();
	pCon->entity = this;
	pCon->type = 0;
	pCon->dead = false;
	fallRigidBody->setUserPointer(pCon);
	PhysicsManager::Instance()->GetDynamicsWorld()->addRigidBody(fallRigidBody);
	D3DXMATRIX identity; 
	D3DXMatrixIdentity( &identity ); 
	matRotation = identity;
	linearVelocity = btVector3(0,0,0);
	//

	/*
	btCollisionShape* groundShape = new btStaticPlaneShape(btVector3(0,1,0),1);
	btDefaultMotionState* groundMotionState = new btDefaultMotionState(btTransform(btQuaternion(0,0,0,1),btVector3(0,-1,0)));
	btRigidBody::btRigidBodyConstructionInfo groundRigidBodyCI(0,groundMotionState,groundShape,btVector3(0,0,0));
	btRigidBody* groundRigidBody = new btRigidBody(groundRigidBodyCI);
	//groundRigidBody->setFriction(btScalar(100.0f));
	dynamicsWorld->addRigidBody(groundRigidBody);*/
}

Cheese::Cheese(D3DXVECTOR3 _position, LPCTSTR mesh_name, LPCTSTR texture_name) : Entity(_position, mesh_name, texture_name)
{
	btConvexHullShape* fallShape = new btConvexHullShape();
	LPDIRECT3DVERTEXBUFFER9 v_buffer;
	mesh->GetVertexBuffer(&v_buffer);
	CUSTOMVERTEX * thisVertex;
	v_buffer->Lock(0,0,(void**)&thisVertex ,0);
	// make collision shape from mesh vertices
	for(int i = 0; i < mesh->GetNumVertices(); i++)
	{
		fallShape->addPoint(btVector3(thisVertex->X, thisVertex->Y, thisVertex->Z));
		thisVertex++;
	}
	v_buffer->Unlock();
	v_buffer->Release();
	btDefaultMotionState* fallMotionState = new btDefaultMotionState(btTransform(btQuaternion(0,0,0,1),btVector3(position.x,position.y,position.z)));
	btScalar mass = 1;
	btVector3 fallInertia(0,-1,0);
	fallShape->calculateLocalInertia(mass,fallInertia);
	btRigidBody::btRigidBodyConstructionInfo fallRigidBodyCI(mass,fallMotionState,fallShape,fallInertia);
	fallRigidBody = new btRigidBody(fallRigidBodyCI);
	//fallRigidBody->setCollisionFlags(btCollisionObject::CF_NO_CONTACT_RESPONSE);
	//fallRigidBody->setCollisionFlags(fallRigidBody->getCollisionFlags() | btCollisionObject::CF_NO_CONTACT_RESPONSE);
	// setup handle to this object (from the collision manifold)
	pCon = new PhysicsContainer();
	pCon->entity = this;
	pCon->type = 3;
	pCon->dead = false;
	fallRigidBody->setUserPointer(pCon);
	//fallRigidBody->setSleepingThresholds (0.0, 0.0);
	//fallRigidBody->setAngularFactor (0.0);
	PhysicsManager::Instance()->GetDynamicsWorld()->addRigidBody(fallRigidBody);
	D3DXMATRIX identity; 
	D3DXMatrixIdentity( &identity ); 
	matRotation = identity;
	linearVelocity = btVector3(0,0,0);
}

Knife::Knife(D3DXVECTOR3 _position, LPCTSTR mesh_name, LPCTSTR texture_name, float _floating_height) : Entity(_position, mesh_name, texture_name)
{
	btConvexHullShape* fallShape = new btConvexHullShape();
	LPDIRECT3DVERTEXBUFFER9 v_buffer;
	mesh->GetVertexBuffer(&v_buffer);
	CUSTOMVERTEX * thisVertex;
	v_buffer->Lock(0,0,(void**)&thisVertex ,0);
	// make collision shape from mesh vertices
	for(int i = 0; i < mesh->GetNumVertices(); i++)
	{
		fallShape->addPoint(btVector3(thisVertex->X, thisVertex->Y, thisVertex->Z));
		thisVertex++;
	}
	v_buffer->Unlock();
	v_buffer->Release();
	btDefaultMotionState* fallMotionState = new btDefaultMotionState(btTransform(btQuaternion(0,0,0,1),btVector3(position.x,position.y,position.z)));
	btScalar mass = 1;
	btVector3 fallInertia(0,-1,0);
	fallShape->calculateLocalInertia(mass,fallInertia);
	btRigidBody::btRigidBodyConstructionInfo fallRigidBodyCI(mass,fallMotionState,fallShape,fallInertia);
	fallRigidBody = new btRigidBody(fallRigidBodyCI);
	//fallRigidBody->setCollisionFlags(btCollisionObject::CF_NO_CONTACT_RESPONSE);
	//fallRigidBody->setCollisionFlags(fallRigidBody->getCollisionFlags() | btCollisionObject::CF_NO_CONTACT_RESPONSE);
	// setup handle to this object (from the collision manifold)
	pCon = new PhysicsContainer();
	pCon->entity = this;
	pCon->type = 2;
	pCon->dead = false;
	fallRigidBody->setUserPointer(pCon);
	fallRigidBody->setSleepingThresholds (0.0, 0.0);
	fallRigidBody->setAngularFactor (0.0);
	PhysicsManager::Instance()->GetDynamicsWorld()->addRigidBody(fallRigidBody);
	D3DXMATRIX identity; 
	D3DXMatrixIdentity( &identity ); 
	matRotation = identity;
	linearVelocity = btVector3(0,0,0);
	fallRigidBody->setLinearFactor(btVector3(0, 1, 0));
	floating_height = _floating_height;
}

void Knife::Update()
{
	if(pCon->dead == true)
	{
		fallRigidBody->setLinearFactor(btVector3(1, 1, 1));
		fallRigidBody->setAngularFactor (1.0);
	}
	if(fallRigidBody->getWorldTransform().getOrigin().getY() < floating_height && pCon->dead == false)
	{
		btVector3 velo = fallRigidBody->getLinearVelocity();
		fallRigidBody->setLinearVelocity(btVector3(velo.getX(), velo.getY()+900, velo.getZ()));
	}
}