#ifndef __ENTITY__
#define __ENTITY__

#include "PhysicsManager.hpp"
#include <d3d9.h>
#include "NeheTimer.hpp"
#include <d3dx9.h>
#include <list>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string>
#include <queue>
#include "EntityManager.hpp"
#include <sstream>
#include "Sound3DObject.hpp"
using namespace std;

class GraphicsManager;

class PhysicsContainer
{
public:
	Entity * entity;
	Missile * missile;
	int type;
	Entity * killer;
	D3DXVECTOR3 soundVector;
	bool dead;
};

class EntityDestination
{
public:
	bool keepMoving;
	D3DXVECTOR3 destination;
	float DistanceFromDestination(D3DXVECTOR3 currentPosition);
	bool movementInProgress;
};

class Particle
{
public:
	Particle(D3DXVECTOR3 _position, D3DXVECTOR3 _direction);
	float life_time_in_seconds;
	float time_at_last_update_seconds;
	float time_at_creation_seconds;
	NeheTimer timer;
	D3DXVECTOR3 position;
	D3DXVECTOR3 direction;
	void Update();
	void Draw();
	bool ShouldDie();
};

class ParticleSystem
{
public:
	LPDIRECT3DTEXTURE9 texture;
	ParticleSystem(D3DXVECTOR3 * _position, D3DXVECTOR3 * _direction, D3DXVECTOR3 _offset); 
	list<Particle> particles;
	D3DXVECTOR3 * position;
	D3DXVECTOR3 offset;
	NeheTimer timer;
	float time_at_last_update_seconds;
	float time_at_last_new_particle;
	D3DXVECTOR3 * direction;
	void Update();
	void Draw();
};

class Entity
{
public:
	static DWORD numMaterials;
	LPDIRECT3DTEXTURE9 texture;
	Sound3DObject * impactSound;
	static LPDIRECT3DTEXTURE9 glow;
	Sound3DObject * movingSound;
	D3DXVECTOR3 facing;
	static GraphicsManager * gMan;
	btRigidBody* fallRigidBody;
	btVector3 linearVelocity;
	ParticleSystem * engine;
	bool dead;
	EntityDestination currentDestination;
	Entity(D3DXVECTOR3 _position, LPCTSTR mesh_name, LPCTSTR texture_name);
	static void InitializeClass();
	virtual void Draw();
	D3DXVECTOR3 position;
	D3DXVECTOR3 vRotation;
	NeheTimer firingTimer;
	float lastFireTime;
	int hit_points;
	string text;
	LPD3DXMESH mesh;
	int faction;
	~Entity();
	void Logic();
	PhysicsContainer * pCon;
	Entity * target;
	D3DXMATRIX matRotation;
	btRigidBody* GetRigidBody();
	virtual void Update();
	void Attack(Entity * _target);
	void Face(D3DXVECTOR3 vec);
	void MoveTo(D3DXVECTOR3 vec);
	void AddLinearVelocity(const btVector3 & velocity);
	void SetPosition(D3DXVECTOR3 _position);

};

class MousePlayer : public Entity
{
public:
	LPD3DXMESH vernin_jump;
	LPD3DXMESH vernin_right;
	LPD3DXMESH vernin_left;
	LPD3DXMESH vernin_stand;
	int frame;
	bool player_moving_right;
	bool player_moving_left;
	MousePlayer(D3DXVECTOR3 _position, LPCTSTR mesh_name, LPCTSTR texture_name);
	virtual void Update();
	virtual void Draw();
	Jump mouseJump;
	Entity * weapon;
	NeheTimer animationTimer;
	float time_since_last_frame;
	int cheese;

};

class Knife : public Entity
{
public:
	Knife(D3DXVECTOR3 _position, LPCTSTR mesh_name, LPCTSTR texture_name, float _floating_height);
	virtual void Update();
	float floating_height;

};

class StaticEntity : public Entity
{
public:
	StaticEntity(D3DXVECTOR3 _position, LPCTSTR mesh_name, LPCTSTR texture_name);
	
};

class Cheese : public Entity
{
public:

	Cheese(D3DXVECTOR3 _position, LPCTSTR mesh_name, LPCTSTR texture_name);
	
};

#endif