#ifndef __FMODUtilityH_
#define __FMODUtilityH_
#pragma comment (lib, "fmodex_vc.lib")
// Standard Includes
#include <windows.h>
#include <stdio.h>
#include <conio.h>

// FMod Inlcudes
#include "fmod.hpp"
#include "fmod_errors.h"

class Sound3DObject;

class FMODManager
{
	public:
		FMOD::System	*system;			//FMod system contains hardware information, allows us to make calls to FMod functions
		FMOD::Sound		*streamedSound;		//Sound to store our streaming music.
		FMOD::Sound     *sounds[7];			//Sound effects.
		FMOD::Channel	*streamedChannel;	//Channel to play our music through.
		FMOD::Channel	*playOnlyOneChannel;//Lets only have one copy of this sound playing at a time.
		FMOD::Channel	*generalChannel;	//Our general channel pointer (used for short sounds that will finish and free up their channel).

		/*NEW*/
		FMOD::Channel	*right3DLoopChannel;	//Channel for the right looping sound
		FMOD::Channel	*moving3DLoopChannel;	//Channel for the moving looping sound.
		/*END NEW*/

		FMOD::DSP		*dspLowPass;		//Our digital signal processing filters
		FMOD::DSP		*dspTremelo;
		FMOD::DSP		*dspChorus;

		FMOD_RESULT       result;		 // Information passed back when you run an FMod function
		unsigned int      version;		 // Current FMod version

		int InitFMOD(int NumChannels);									// Contains all of our initializations for FMod
		void CreateSound(const char *soundPath, FMOD::Sound *&sound, 
			bool isStream, FMOD_MODE createFlags = NULL);


		FMOD_VECTOR listenerPos;
		FMOD_VECTOR listenerVel;
		FMOD_VECTOR forward;
		FMOD_VECTOR up;

		void PlayFSound(FMOD::Sound *sound, FMOD::Channel *&channel, bool reuseChannel = false);
		void PauseAllChannels();
		void StopAllChannels();
		void PauseChannelToggle(FMOD::Channel *&channel);
		void PauseChannel(FMOD::Channel *&channel);
		void UnpauseChannel(FMOD::Channel *&channel);
		void CreateDSP(FMOD_DSP_TYPE dspType, FMOD::DSP *&curDSP);
		void SetDSP(FMOD::DSP *&curDSP);
		void RemoveDSP(FMOD::DSP *&curDSP);
		void ToggleDSP(FMOD::DSP *&curDSP);
		void SetListenerPosition(float xPos, float yPos, float zPos);
		void ChannelUpdate(FMOD::Sound *sound);							// Display current information about sound channels and sounds
};

namespace FMODUtil
{
	void ERRCHECK(FMOD_RESULT result);	// Check the result passed back by FMod functions for obvious errors
}

#endif // __FMODUtilityH_