#include "GraphicsManager.hpp"


GraphicsManager* GraphicsManager::sInstance = NULL;
LPDIRECT3D9 GraphicsManager::d3d = NULL;
LPDIRECT3DDEVICE9 GraphicsManager::d3ddev = NULL;
CCamera GraphicsManager::camHandler;
LPDIRECT3DVERTEXBUFFER9 GraphicsManager::v_buffer = NULL;
LPDIRECT3DINDEXBUFFER9 GraphicsManager::i_buffer = NULL;
vector<Ray> GraphicsManager::rays;
Entity * GraphicsManager::selectedEntity = NULL;
LPD3DXFONT GraphicsManager::font = NULL;
HDC GraphicsManager::hdc = NULL;
DWORD GraphicsManager::FrameCnt = 0; // The number of frames that have occurred.
float GraphicsManager::TimeElapsed = 0; // The time that has elapsed so far.
float GraphicsManager::FPS = 0; // The frames rendered per second.
NeheTimer GraphicsManager::neheTimer;
LPD3DXMESH GraphicsManager::star_mesh;
LPDIRECT3DTEXTURE9 GraphicsManager::star_texture;
MousePlayer * GraphicsManager::player = NULL;
LPDIRECT3DTEXTURE9 GraphicsManager::VernonTexture;
LPDIRECT3DTEXTURE9 GraphicsManager::CheeseTexture;
LPDIRECT3DTEXTURE9 GraphicsManager::ClockTexture;
LPDIRECT3DTEXTURE9 GraphicsManager::HealthTexture;
LPDIRECT3DTEXTURE9 GraphicsManager::CollectTexture;
LPDIRECT3DTEXTURE9 GraphicsManager::DefaultTexture;
LPD3DXSPRITE GraphicsManager::sprite;
LPD3DXSPRITE GraphicsManager::sprite2;
LPD3DXSPRITE GraphicsManager::sprite3;
LPD3DXSPRITE GraphicsManager::sprite4;
LPD3DXSPRITE GraphicsManager::sprite5;
LPD3DXSPRITE GraphicsManager::sprite6;
LPD3DXSPRITE GraphicsManager::sprite7;


//these globals are adjusted depending on how much life Vernon has left,
//and how much cheese he has collected. as these numbers decrease, the
//corresponding health bars at the top of the screen will shrink
float health			= 15.0f;
float collected			= 1.0f;
const float healthMAX	= 15.0f;
const float collectedMAX= 15.0f;

GraphicsManager* GraphicsManager::Instance()
{
	if(!sInstance)
		sInstance = new GraphicsManager();
	return sInstance;
}

void GraphicsManager::Initialize(HWND hWnd)
{
	int vertexCount =	24;		//For now well modify the size of our vertex and index buffers
	int indexCount =	36;		//with these variables.

    d3d = Direct3DCreate9(D3D_SDK_VERSION);    // create the Direct3D interface (COM object)
	//D3D_SDK_VERSION passes our current version of the SDK to the function.
	//This tells the users DirectX which version you developed your game for.
	//So when a user with a more up to date version of DX runs your program, it will execute your
	//So if a user has a more up to date version of DX, your application will only use features from
	//the version you developed for.

    D3DPRESENT_PARAMETERS d3dpp;    // create a struct to hold various device information from drivers to actual hardware.
									//This struct can contain a lot of specific flags and features, but well only address a few.

    ZeroMemory(&d3dpp, sizeof(d3dpp));    // clear out the struct for use (So we don't have to set each individual struct member to null.).
    d3dpp.Windowed = TRUE;    // program windowed, not fullscreen
    d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;   // discard old frames (There are 3 different swap effects, this is the fastest).
												// This drops old back buffers, other swap effects may need to be used in its place for different special effects.
    d3dpp.hDeviceWindow = hWnd;    // set the window to be used by Direct3D
    d3dpp.BackBufferFormat = D3DFMT_X8R8G8B8;
    d3dpp.BackBufferWidth = 1680;
    d3dpp.BackBufferHeight = 1050;

	d3dpp.EnableAutoDepthStencil = TRUE;	//Tell d3d to automatically create our zbuffer, and set it up in the most common way.
	d3dpp.AutoDepthStencilFormat = D3DFMT_D16;	//Format for each pixel in the zbuffer.
	d3dpp.PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;


    // create a device class using this information and the info from the d3dpp stuct
    d3d->CreateDevice(D3DADAPTER_DEFAULT,	//Specify the graphics card we're using (since there's usually only one, lets just go with the default card).
                      D3DDEVTYPE_HAL,		//Specifies that we should use the hardware abstraction layer (use hardware to process graphics instead of software).
											//If for some reason the hardware cannot render something, software will be used instead.
                      hWnd,					//Handle to our window.
                      D3DCREATE_SOFTWARE_VERTEXPROCESSING, //The slowest of the three flags we can use here, using for compatibilities sake.
                      &d3dpp,				//Pointer to the struct we filled in earlier.
                      &d3ddev);				//Pointer to return the device interface to.

	d3ddev->SetRenderState(D3DRS_LIGHTING, TRUE);		// turn off the 3D lighting
	d3ddev->SetRenderState(D3DRS_ZENABLE, TRUE);		// turn on the z-buffer
	d3ddev->SetRenderState(D3DRS_MULTISAMPLEANTIALIAS , TRUE);

	//d3ddev->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME );	//Turn on wire
	//d3ddev->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
	camHandler.Position_Camera(
		0.0f, 2000.0f, -3000.0f, 
		0.0f, 1000.0f, 0.0f, 
		0.0f, 1.0f, 0.0f);
	hdc = CreateCompatibleDC( 0 );
	Init2DText();
	Init3DText();
	neheTimer.Initialize();

	DWORD numMaterials;
	LPD3DXBUFFER bufMeshMaterial;
	D3DXLoadMeshFromX("stars.x", 
		D3DXMESH_SYSTEMMEM,
		d3ddev,
		NULL,
		&bufMeshMaterial,
		NULL,
		&numMaterials,
		&star_mesh);
	D3DXCreateTextureFromFile(d3ddev,    // the Direct3D device
                              "stars.jpg",    // the filename of the texture
                              &star_texture);			// the address of the texture storage
	D3DMATERIAL9 defaultmaterial;
	::ZeroMemory(&defaultmaterial, sizeof(defaultmaterial));
	defaultmaterial.Diffuse = D3DXCOLOR(1.0f, 1.0f, 1.0f, 1.0f); // red
	defaultmaterial.Ambient = D3DXCOLOR(1.0f, 1.0f, 1.0f, 1.0f); // red
	defaultmaterial.Specular = D3DXCOLOR(1.0f, 1.0f, 1.0f, 1.0f); // red
	defaultmaterial.Emissive = D3DXCOLOR(0.0f, 0.0f, 0.0f, 1.0f); // no emission
	defaultmaterial.Power = 5.0f;

	d3ddev->SetMaterial(&defaultmaterial);

	D3DLIGHT9 dir;
	::ZeroMemory(&dir, sizeof(dir));
	dir.Type = D3DLIGHT_DIRECTIONAL;
	dir.Diffuse = D3DXCOLOR(1.0f, 1.0f, 1.0f, 1.0f);
	dir.Specular = D3DXCOLOR(0.3f, 0.3f, 0.3f, 0.3f);
	dir.Ambient = D3DXCOLOR(0.6f, 0.6f, 0.6f, 0.6f);
	dir.Direction = D3DXVECTOR3(-0.5f, -1.0f, 0.0f);
	d3ddev->SetLight(0, &dir);
	d3ddev->LightEnable(0, true);
	d3ddev->SetRenderState(D3DRS_NORMALIZENORMALS, true);
	d3ddev->SetRenderState(D3DRS_SPECULARENABLE, true);

	//DOIN SHI-AAT
	//alpha blending enabled
	d3ddev->SetRenderState(D3DRS_ALPHABLENDENABLE,true);
	//source blend factor
	d3ddev->SetRenderState(D3DRS_SRCBLEND,D3DBLEND_SRCALPHA);
	//destination blend factor
	d3ddev->SetRenderState(D3DRS_DESTBLEND,D3DBLEND_INVSRCALPHA);
	//alpha from texture
	d3ddev->SetTextureStageState(0,D3DTSS_ALPHAARG1,D3DTA_TEXTURE);

	D3DXCreateTextureFromFileExA(d3ddev, "Images/Vernon.png", D3DX_DEFAULT, D3DX_DEFAULT, D3DX_DEFAULT, NULL, D3DFMT_UNKNOWN,
		D3DPOOL_MANAGED, D3DX_DEFAULT, D3DX_DEFAULT, 0, NULL, NULL, &VernonTexture);

	D3DXCreateTextureFromFileExA(d3ddev, "Images/Cheese.png", D3DX_DEFAULT, D3DX_DEFAULT, D3DX_DEFAULT, NULL, D3DFMT_UNKNOWN,
		D3DPOOL_MANAGED, D3DX_DEFAULT, D3DX_DEFAULT, 0, NULL, NULL, &CheeseTexture);

	D3DXCreateTextureFromFileExA(d3ddev, "Images/Clock.png", D3DX_DEFAULT, D3DX_DEFAULT, D3DX_DEFAULT, NULL, D3DFMT_UNKNOWN,
		D3DPOOL_MANAGED, D3DX_DEFAULT, D3DX_DEFAULT, 0, NULL, NULL, &ClockTexture);

	D3DXCreateTextureFromFileExA(d3ddev, "Images/health-o-meter.png", D3DX_DEFAULT, D3DX_DEFAULT, D3DX_DEFAULT, NULL, D3DFMT_UNKNOWN,
		D3DPOOL_MANAGED, D3DX_DEFAULT, D3DX_DEFAULT, 0, NULL, NULL, &HealthTexture);

	D3DXCreateTextureFromFileExA(d3ddev, "Images/cheese-o-meter.png", D3DX_DEFAULT, D3DX_DEFAULT, D3DX_DEFAULT, NULL, D3DFMT_UNKNOWN,
		D3DPOOL_MANAGED, D3DX_DEFAULT, D3DX_DEFAULT, 0, NULL, NULL, &CollectTexture);

	D3DXCreateTextureFromFileExA(d3ddev, "Images/default-o-meter.png", D3DX_DEFAULT, D3DX_DEFAULT, D3DX_DEFAULT, NULL, D3DFMT_UNKNOWN,
		D3DPOOL_MANAGED, D3DX_DEFAULT, D3DX_DEFAULT, 0, NULL, NULL, &DefaultTexture);

	//D3DXCreateTextureFromFile(d3ddev, "Images/Vernon.jpg", &VernonTexture);
	//D3DXCreateTextureFromFile(d3ddev, "Images/Cheese.png", &CheeseTexture);
	//D3DXCreateTextureFromFile(d3ddev, "Images/Clock.jpg", &ClockTexture);
	//D3DXCreateTextureFromFile(d3ddev, "Images/heart-o-meter.jpg", &HealthTexture);
	//D3DXCreateTextureFromFile(d3ddev, "Images/cheese-o-meter.jpg", &CollectTexture);

	D3DXCreateSprite(d3ddev, &sprite);
	D3DXCreateSprite(d3ddev, &sprite2);
	D3DXCreateSprite(d3ddev, &sprite3);
	D3DXCreateSprite(d3ddev, &sprite4);
	D3DXCreateSprite(d3ddev, &sprite5);
	D3DXCreateSprite(d3ddev, &sprite6);
	D3DXCreateSprite(d3ddev, &sprite7);

}


void GraphicsManager::Draw()
{
	// clear the window to a deep blue
    d3ddev->Clear(0,				//Number of rectangles used in the array of the second parameter
		NULL,						//Array of rectangles we want to clear (We'll never be using these first two params).
		D3DCLEAR_TARGET,			//Flag f owhat we want to clear (this flag specifies the backbuffer).
		D3DCOLOR_XRGB(0, 0, 0),	//Colour to clear to.
		1.0f,						//Depth
		0);							
	d3ddev->Clear(0, NULL, D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB(0, 0, 0), 1.0f, 0); //Same as above but targetting the zbuffer.

    d3ddev->BeginScene();   // begins the 3D scene

	    // select which vertex format we are using
        d3ddev->SetFVF(CUSTOMFVF);

		//************************************************
		//				Begin World Space				//
		//************************************************
	    
		// draw entities
		EntityManager * eMan = EntityManager::Instance();
		
		for(int i = 0; i < eMan->entities.size(); i++)
		{
			if(eMan->entities[i] != NULL)
			{
				if(eMan->entities[i]->pCon->type != 3)
					eMan->entities[i]->Draw();
				else if(eMan->entities[i]->pCon->dead == false)
					eMan->entities[i]->Draw();
			}

		}
		player->Draw();
		if(player->weapon != NULL)
			player->weapon->Draw();
		
		// draw the axes
		d3ddev->SetRenderState(D3DRS_LIGHTING, FALSE);
		//DrawLine(D3DXVECTOR3(-3000.0f,0.0f,0.0f), D3DXVECTOR3(3000.0f,0.0f,0.0f), D3DCOLOR_XRGB(255, 0, 0));
		//DrawLine(D3DXVECTOR3(0.0f,-3000.0f,0.0f), D3DXVECTOR3(0.0f,3000.0f,0.0f), D3DCOLOR_XRGB(0, 255, 0));
		//DrawLine(D3DXVECTOR3(0.0f,0.0f,-3000.0f), D3DXVECTOR3(0.0f,0.0f,3000.0f), D3DCOLOR_XRGB(0, 0, 255));
		d3ddev->SetRenderState(D3DRS_LIGHTING, TRUE);

	
		HUDhelper();
		HUD();
		
		//************************************************
		//				End World Space					//
		//************************************************

		d3ddev->SetRenderState(D3DRS_NORMALIZENORMALS, true);
		//************************************************
		//				Begin View Space				//
		//************************************************

		D3DXMATRIX matView;    // the view transform matrix

		D3DXMatrixLookAtLH(&matView,	// Update View Matrix
			  &D3DXVECTOR3(camHandler.mPos.x,  camHandler.mPos.y,  camHandler.mPos.z),	
			  &D3DXVECTOR3(camHandler.mView.x, camHandler.mView.y, camHandler.mView.z),	
			  &D3DXVECTOR3(camHandler.mUp.x,   camHandler.mUp.y,   camHandler.mUp.z) );

		d3ddev->SetTransform(D3DTS_VIEW, &matView);    // set the view transform to matView

		//************************************************
		//				End View Space					//
		//************************************************

		//************************************************
		//				Begin Projection Space			//
		//************************************************

		D3DXMATRIX matProjection;     // the projection transform matrix

		D3DXMatrixPerspectiveFovLH(&matProjection,
								   D3DXToRadian(45),    // the horizontal field of view
								   1680.0f / 1050.0f, // aspect ratio
								   1.0f,    // the near view-plane
								   100000.0f);    // the far view-plane

		d3ddev->SetTransform(D3DTS_PROJECTION, &matProjection);    // set the projection

		//************************************************
		//				End Projection Space			//
		//************************************************

	d3ddev->EndScene();     // ends the 3D scene

	d3ddev->Present(		// displays the created frame on the screen
		NULL, NULL, NULL, NULL);	//Three of these parameters deal with the slowest flag for the swap chain, 
									//and the other refers to drawing to another window.
}

void GraphicsManager::DrawLine(D3DXVECTOR3 start, D3DXVECTOR3 finish, DWORD lineColor)
{
	D3DXMATRIX identity; 
	D3DXMatrixIdentity( &identity ); 
	d3ddev->SetTransform(D3DTS_WORLD, &(identity));
	d3ddev->SetTexture(0, NULL);
	d3ddev->SetFVF(LINEFVF);
		
	LINEVERTEX vertices[2] =
	{
		{ start.x, start.y, start.z,  lineColor },    // side 1
		{ finish.x, finish.y, finish.z,  lineColor },
	};

	d3ddev->DrawPrimitiveUP(D3DPT_LINELIST,         //PrimitiveType
		1,              //PrimitiveCount
		vertices,            //pVertexStreamZeroData
		sizeof(LINEVERTEX)	);   //VertexStreamZeroStride

	d3ddev->SetFVF(CUSTOMFVF);
}

void GraphicsManager::DrawPoint(D3DXVECTOR3 start)
{
	float fPointSize = 0.5f, fPointScaleB = 0.5f;
	d3ddev->SetRenderState(D3DRS_POINTSPRITEENABLE,true);
	d3ddev->SetRenderState(D3DRS_POINTSCALEENABLE,true);
	d3ddev->SetRenderState(D3DRS_POINTSIZE,
                                  *((DWORD*)&fPointSize));
	d3ddev->SetRenderState(D3DRS_POINTSCALE_B,
									  *((DWORD*)&fPointScaleB));

	d3ddev->SetRenderState(D3DRS_ALPHABLENDENABLE,true);
	d3ddev->SetRenderState( D3DRS_ZWRITEENABLE, FALSE );
	d3ddev->SetRenderState(D3DRS_SRCBLEND,D3DBLEND_ONE);
	d3ddev->SetRenderState(D3DRS_DESTBLEND,D3DBLEND_ONE);



	D3DXMATRIX identity; 
	D3DXMatrixIdentity( &identity ); 
	d3ddev->SetTransform(D3DTS_WORLD, &(identity));
	d3ddev->SetFVF(SPRITEFVF);
		
	SPRITEVERTEX vertices[1] =
	{
		{ start.x, start.y, start.z,  0.5f, D3DCOLOR_XRGB(255, 0, 0)},    // side 1
	};

	d3ddev->DrawPrimitiveUP(D3DPT_POINTLIST,         //PrimitiveType
		1,              //PrimitiveCount
		vertices,            //pVertexStreamZeroData
		sizeof(SPRITEFVF)	);   //VertexStreamZeroStride

	d3ddev->SetFVF(CUSTOMFVF);
	d3ddev->SetRenderState(D3DRS_POINTSPRITEENABLE,false);
	d3ddev->SetRenderState(D3DRS_POINTSCALEENABLE,false);
	d3ddev->SetRenderState( D3DRS_ZWRITEENABLE, TRUE );
	d3ddev->SetRenderState(D3DRS_ALPHABLENDENABLE,false);
}

// Based on code from Introduction to 3d Game Programming With DirectX 9.0 by Frank Luna
Ray GraphicsManager::CalcPickingRay(int x, int y)
{
	float px = 0.0f;
	float py = 0.0f;

	D3DVIEWPORT9 vp;
	d3ddev->GetViewport(&vp);

	D3DXMATRIX proj;
	d3ddev->GetTransform(D3DTS_PROJECTION, &proj);

	px = ((( 2.0f*x) / vp.Width)  - 1.0f) / proj(0, 0);
	py = (((-2.0f*y) / vp.Height) + 1.0f) / proj(1, 1);

	Ray ray;
	ray._origin    = D3DXVECTOR3(0.0f, 0.0f, 0.0f);
	ray._direction = D3DXVECTOR3(px, py, 1.0f);

	return ray;
}

// Based on code from Introduction to 3d Game Programming With DirectX 9.0 by Frank Luna
void GraphicsManager::TransformRay(Ray* ray, D3DXMATRIX* T)
{
	// transform the ray's origin, w = 1.
	D3DXVec3TransformCoord(
		&ray->_origin,
		&ray->_origin,
		T);

	// transform the ray's direction, w = 0.
	D3DXVec3TransformNormal(
		&ray->_direction,
		&ray->_direction,
		T);

	// normalize the direction
	D3DXVec3Normalize(&ray->_direction, &ray->_direction);
}

// Based on code from Introduction to 3d Game Programming With DirectX 9.0 by Frank Luna
void GraphicsManager::Init2DText()
{
	D3DXFONT_DESC FontDesc = {24,
                          0,
                          400,
                          0,
                          false,
                          DEFAULT_CHARSET,
                          OUT_TT_PRECIS,
                          CLIP_DEFAULT_PRECIS,
                          DEFAULT_PITCH,
                          "Arial"};
	D3DXCreateFontIndirect(d3ddev,&FontDesc,&font);
}

// Based on code from Introduction to 3d Game Programming With DirectX 9.0 by Frank Luna
void GraphicsManager::Init3DText()
{
	// Obtain a handle to a device context.
	//HDC hdc = CreateCompatibleDC( 0 );
	// Fill out a LOGFONT structure that describes the fonts properties.
	LOGFONT lf;
	ZeroMemory(&lf, sizeof(LOGFONT));
	lf.lfHeight = 25; // in logical units
	lf.lfWidth = 12; // in logical units
	lf.lfWeight = 0; // boldness, range 0(light) - 1050(bold)
	lf.lfItalic = false;
	lf.lfUnderline = false;
	lf.lfStrikeOut = false;
	lf.lfCharSet = DEFAULT_CHARSET;
	strcpy(lf.lfFaceName, "Times New Roman"); // font style
	// Create a font and select that font with the device context.
	HFONT hFont;
	HFONT hFontOld;
	hFont = CreateFontIndirect(&lf);
	hFontOld = (HFONT)SelectObject(hdc, hFont);
}

// Based on code from Introduction to 3d Game Programming With DirectX 9.0 by Frank Luna
void GraphicsManager::Draw3DText(LPCTSTR text) 
{
	ID3DXMesh* Text = 0;
	D3DXCreateText(d3ddev, hdc, text, 0.001f, 0.001f, &Text, 0, 0);
	Text->DrawSubset(0);
}

// Based on code from Introduction to 3d Game Programming With DirectX 9.0 by Frank Luna
void GraphicsManager::Draw2DText(LPCTSTR text, LONG StartX, LONG StartY, LONG EndX, LONG EndY)
{
		RECT FontPosition;

		FontPosition.left	= StartX;
		FontPosition.top	= StartY;
		FontPosition.right	= EndX;
		FontPosition.bottom = EndY;

		font->DrawText(NULL,
                   text,
                   -1,
                   &FontPosition,
                   DT_LEFT,
                   0xffff0000); //draw text
}

// Based on code from Introduction to 3d Game Programming With DirectX 9.0 by Frank Luna
void GraphicsManager::CalcFPS(float timeDelta)
{
	FrameCnt++;
	TimeElapsed += timeDelta;
	if(TimeElapsed >= 1.0f)
	{
		FPS = (float)FrameCnt / TimeElapsed;
		TimeElapsed = 0.0f;
		FrameCnt = 0;
	}
}

void GraphicsManager::HUD()
{
	static LPCTSTR temp;
	health = player->hit_points;
	//Vernon's health
	temp = "Health:";
	Draw2DText(temp, 240, 20, 1680, 1050);

	//accumulated cheese
	temp = "Cheese:";
	Draw2DText(temp, 226, 50, 1680, 1050);

	//current score
	temp = "Score:";
	Draw2DText(temp, 900, 20, 1680, 1050);

	//time remaining
	static int timeRemaining = 300;
	static float timeSinceLastUpdate = 0;
	static float timeSinceLastChange = 0;

	neheTimer.Update();
	timeSinceLastUpdate = neheTimer.Get_dt();
	timeSinceLastChange += timeSinceLastUpdate;

	//if a keyboard button has been pressed or 3 seconds have elapsed
	if(timeSinceLastChange >= 100.00f)
	{
		timeRemaining --;
		timeSinceLastChange = timeSinceLastUpdate = 0.0f;
		//health -= .25;
	}

	if(timeRemaining <= 0)
	{
		//time is up
		temp = "TIME UP!";
		Draw2DText(temp, 1420, 20, 1680, 1050);
	}

	else
	{
		char tempString [50];
		sprintf(tempString, "Time: %d\n", timeRemaining);
		OutputDebugStr(tempString);

		//temp = "Time: %d";
		Draw2DText(tempString, 1420, 20, 1680, 1050);
		
	}
}

void GraphicsManager::HUDhelper()
{
	/* *** ***** *******VERNON******* ***** *** */
	sprite->Begin(D3DXSPRITE_ALPHABLEND);

	//Texture being used is 32 by 32:
	D3DXVECTOR2 spriteCentre=D3DXVECTOR2(16.0f, 16.0f);

	//Screen position of the sprite
	D3DXVECTOR2 trans=D3DXVECTOR2(140.0f, 0.0f);

	// Rotate based on the time passed
	float rotation = 00.0f;

	// Build our matrix to rotate, scale and position our sprite
	D3DXMATRIX mat;
	D3DXVECTOR2 scaling(2.1f, 2.1f);

	// out, scaling centre, scaling rotation, scaling, rotation centre, rotation, translation
	D3DXMatrixTransformation2D(&mat, NULL,0.0, &scaling, &spriteCentre, rotation, &trans);

	// Tell the sprite about the matrix
	sprite->SetTransform(&mat);

	// Draw the sprite 
	sprite->Draw(VernonTexture, NULL, NULL, NULL, 0xFFFFFFFF);

	// Thats it
	sprite->End();


	/* *** ***** *******CHEESE******* ***** *** */
	sprite2->Begin(NULL);

	//Texture being used is 32 by 32:
	D3DXVECTOR2 sprite2Centre = D3DXVECTOR2(16.0f, 16.0f);

	//Screen position of the sprite
	D3DXVECTOR2 trans2 = D3DXVECTOR2(155.0f, 40.0f);

	// Rotate based on the time passed
	float rotation2 = 00.0f;

	// Build our matrix to rotate, scale and position our sprite
	D3DXMATRIX mat2;
	D3DXVECTOR2 scaling2(1.8f, 1.8f);

	// out, scaling centre, scaling rotation, scaling, rotation centre, rotation, translation
	D3DXMatrixTransformation2D(&mat2, NULL, 0.0, &scaling2, &sprite2Centre, rotation2, &trans2);

	// Tell the sprite about the matrix
	sprite2->SetTransform(&mat2);

	// Draw the sprite 
	sprite2->Draw(CheeseTexture, NULL, NULL, NULL, 0xFFFFFFFF);

	// Thats it
	sprite2->End();


	/* *** ***** *******CLOCK******* ***** *** */
	sprite3->Begin(NULL);

	//Texture being used is 32 by 32:
	D3DXVECTOR2 sprite3Centre = D3DXVECTOR2(16.0f, 16.0f);

	//Screen position of the sprite
	D3DXVECTOR2 trans3 = D3DXVECTOR2(1350.0f, 0.0f);

	// Rotate based on the time passed
	float rotation3 = 00.0f;

	// Build our matrix to rotate, scale and position our sprite
	D3DXMATRIX mat3;
	D3DXVECTOR2 scaling3(2.2f, 2.2f);

	// out, scaling centre, scaling rotation, scaling, rotation centre, rotation, translation
	D3DXMatrixTransformation2D(&mat3, NULL, 0.0, &scaling3, &sprite3Centre, rotation3, &trans3);

	// Tell the sprite about the matrix
	sprite3->SetTransform(&mat3);

	// Draw the sprite 
	sprite3->Draw(ClockTexture, NULL, NULL, NULL, 0xFFFFFFFF);

	// Thats it
	sprite3->End();


	/* *** ***** *******HEALTH******* ***** *** */
	sprite4->Begin(D3DXSPRITE_ALPHABLEND);

	//Texture being used is 32 by 32:
	D3DXVECTOR2 sprite4Centre = D3DXVECTOR2(16.0f, 16.0f);

	//Screen position of the sprite
	D3DXVECTOR2 trans4 = D3DXVECTOR2(340.0f, 5.0f);

	// Rotate based on the time passed
	float rotation4 = 00.0f;

	// Build our matrix to rotate, scale and position our sprite
	D3DXMATRIX mat4;
	D3DXVECTOR2 scaling4(health, 2.0f);

	// out, scaling centre, scaling rotation, scaling, rotation centre, rotation, translation
	D3DXMatrixTransformation2D(&mat4, NULL, 0.0, &scaling4, &sprite4Centre, rotation4, &trans4);

	// Tell the sprite about the matrix
	sprite4->SetTransform(&mat4);

	// Draw the sprite 
	sprite4->Draw(HealthTexture, NULL, NULL, NULL, 0xFFFFFFFF);

	// Thats it
	sprite4->End();


	/* *** ***** *******COLLECTED******* ***** *** */
	sprite5->Begin(NULL);

	//Texture being used is 32 by 32:
	D3DXVECTOR2 sprite5Centre = D3DXVECTOR2(16.0f, 16.0f);

	//Screen position of the sprite
	D3DXVECTOR2 trans5 = D3DXVECTOR2(340.0f, 37.0f);

	// Rotate based on the time passed
	float rotation5 = 00.0f;

	// Build our matrix to rotate, scale and position our sprite
	D3DXMATRIX mat5;
	D3DXVECTOR2 scaling5(collected, 2.0f);

	// out, scaling centre, scaling rotation, scaling, rotation centre, rotation, translation
	D3DXMatrixTransformation2D(&mat5, NULL, 0.0, &scaling5, &sprite5Centre, rotation5, &trans5);

	// Tell the sprite about the matrix
	sprite5->SetTransform(&mat5);

	// Draw the sprite 
	sprite5->Draw(CollectTexture, NULL, NULL, NULL, 0xFFFFFFFF);

	// Thats it
	sprite5->End();


	/* *** ***** *******MAX HEALTH******* ***** *** */
	sprite6->Begin(D3DXSPRITE_ALPHABLEND);

	//Texture being used is 32 by 32:
	D3DXVECTOR2 sprite6Centre = D3DXVECTOR2(16.0f, 16.0f);

	//Screen position of the sprite
	D3DXVECTOR2 trans6 = D3DXVECTOR2(340.0f, 5.0f);

	// Rotate based on the time passed
	float rotation6 = 00.0f;

	// Build our matrix to rotate, scale and position our sprite
	D3DXMATRIX mat6;
	D3DXVECTOR2 scaling6(healthMAX, 2.0f);

	// out, scaling centre, scaling rotation, scaling, rotation centre, rotation, translation
	D3DXMatrixTransformation2D(&mat6, NULL, 0.0, &scaling6, &sprite6Centre, rotation6, &trans6);

	// Tell the sprite about the matrix
	sprite6->SetTransform(&mat6);

	// Draw the sprite 
	sprite6->Draw(DefaultTexture, NULL, NULL, NULL, 0xFFFFFFFF);

	// Thats it
	sprite6->End();


	/* *** ***** *******MAX COLLECTED******* ***** *** */
	sprite7->Begin(D3DXSPRITE_ALPHABLEND);

	//Texture being used is 32 by 32:
	D3DXVECTOR2 sprite7Centre = D3DXVECTOR2(16.0f, 16.0f);

	//Screen position of the sprite
	D3DXVECTOR2 trans7 = D3DXVECTOR2(340.0f, 37.0f);

	// Rotate based on the time passed
	float rotation7 = 00.0f;

	// Build our matrix to rotate, scale and position our sprite
	D3DXMATRIX mat7;
	D3DXVECTOR2 scaling7(collectedMAX, 2.0f);

	// out, scaling centre, scaling rotation, scaling, rotation centre, rotation, translation
	D3DXMatrixTransformation2D(&mat7, NULL, 0.0, &scaling7, &sprite7Centre, rotation7, &trans7);

	// Tell the sprite about the matrix
	sprite7->SetTransform(&mat7);

	// Draw the sprite 
	sprite7->Draw(DefaultTexture, NULL, NULL, NULL, 0xFFFFFFFF);

	// Thats it
	sprite7->End();
}