#ifndef __GRAPHICSMANAGER__
#define __GRAPHICSMANAGER__

#include <stdlib.h>
#include <d3d9.h>
//This header is not included normally, we must navigate to our SDK include folder in our Vis Studio Settings.
#include <d3dx9.h>
#include "CCamera.hpp"
#include "EntityManager.hpp"
#include "Entity.hpp"
#include "MiscUtilities.hpp"
#include "NeheTimer.hpp"

#define NULL 0
// include the Direct3D Library file
//This can be used to avoid having to place an additional dependency in your project linker. (You can do this with your sdl projects.)
//This makes libraries that may be missing for another developer more visible.
#pragma comment (lib, "d3d9.lib")
//Same as the header, we must include our libraries 
#pragma comment (lib, "d3dx9.lib")

// normal vertex for primative drawing
struct CUSTOMVERTEX {FLOAT X, Y, Z; D3DVECTOR NORMAL; FLOAT U, V;};
#define CUSTOMFVF (D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_TEX1) 
// vertex for line drawing
struct LINEVERTEX {FLOAT X, Y, Z; DWORD colour;};
#define LINEFVF (D3DFVF_XYZ | D3DFVF_DIFFUSE) 
// vertex for point sprite drawing
struct SPRITEVERTEX {FLOAT X, Y, Z; D3DVECTOR NORMAL; FLOAT U, V;};
#define SPRITEFVF (D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_TEX1) 

struct Ray
{
	D3DXVECTOR3 _origin;
	D3DXVECTOR3 _direction;
};



class GraphicsManager
{
private:

public:
	static vector<Ray> rays;
	static CCamera camHandler;
	static LPDIRECT3D9 d3d;
	static MousePlayer * player;
	static LPDIRECT3DDEVICE9 d3ddev;
	static GraphicsManager *sInstance;
	static void Initialize(HWND hWnd);
	static GraphicsManager* Instance();
	static void Draw();
	static LPDIRECT3DVERTEXBUFFER9 v_buffer;
	static LPDIRECT3DINDEXBUFFER9 i_buffer;
	static void DrawLine(D3DXVECTOR3 start, D3DXVECTOR3 finish, DWORD lineColor);
	static void DrawPoint(D3DXVECTOR3 start);
	static void DrawAxes();
	static Ray CalcPickingRay(int x, int y);
	static void TransformRay(Ray* ray, D3DXMATRIX* T);
	static Entity * selectedEntity;
	static LPD3DXFONT font;
	static HDC hdc;
	static void Init2DText();
	static void Init3DText();
	static void Draw3DText(LPCTSTR text);
	static void Draw2DText(LPCTSTR text, LONG StartX, LONG StartY, LONG EndX, LONG EndY);
	static DWORD FrameCnt; // The number of frames that have occurred.
	static float TimeElapsed; // The time that has elapsed so far.
	static float FPS; // The frames rendered per second.
	static void CalcFPS(float timeDelta);
	static NeheTimer neheTimer;
		static LPD3DXMESH star_mesh;
	static LPDIRECT3DTEXTURE9 star_texture;
	static void HUD();
	static void HUDhelper();
		//Texture for vernon
	static LPDIRECT3DTEXTURE9 VernonTexture;
	static LPDIRECT3DTEXTURE9 CheeseTexture;
	static LPDIRECT3DTEXTURE9 ClockTexture;
	static LPDIRECT3DTEXTURE9 HealthTexture;
	static LPDIRECT3DTEXTURE9 CollectTexture;
	static LPDIRECT3DTEXTURE9 DefaultTexture;
	static LPD3DXSPRITE sprite;
	static LPD3DXSPRITE sprite2;
	static LPD3DXSPRITE sprite3;
	static LPD3DXSPRITE sprite4;
	static LPD3DXSPRITE sprite5;
	static LPD3DXSPRITE sprite6;
	static LPD3DXSPRITE sprite7;
};

#endif