#include "PhysicsManager.hpp"
#include "Entity.hpp"
#include "EntityManager.hpp"
#include "GraphicsManager.hpp"

btDiscreteDynamicsWorld* PhysicsManager::dynamicsWorld = NULL;
PhysicsManager* PhysicsManager::sInstance = NULL;
Sound3DObject* PhysicsManager::fireSound = NULL;
MousePlayer * PhysicsManager::player = NULL;

Jump::Jump()
{
	isJumping = false;
	initial_force_of_jump = 300000;
	rate_of_decrease_per_second = 10000;
	current_force = 0;
	jumpTimer.Initialize();
}

void Jump::doJump()
{
	if(isJumping == false)
	{
		isJumping = true;
		current_force = initial_force_of_jump;
	}
}

void Jump::Update()
{
	jumpTimer.Update();
	float dt = jumpTimer.Get_dt();
	if(isJumping)
		current_force -= rate_of_decrease_per_second*dt;
	if(current_force <= 0.0f)
	{
		isJumping = false;
		current_force = 0.0f;
	}

}

PhysicsManager* PhysicsManager::Instance()
{
	if(!sInstance)
		sInstance = new PhysicsManager();
	return sInstance;
}



void PhysicsManager::Initialize()
{
	btBroadphaseInterface* broadphase = new btDbvtBroadphase();

	btDefaultCollisionConfiguration* collisionConfiguration = new btDefaultCollisionConfiguration();
	btCollisionDispatcher* dispatcher = new btCollisionDispatcher(collisionConfiguration);

	btSequentialImpulseConstraintSolver* solver = new btSequentialImpulseConstraintSolver;

	dynamicsWorld = new btDiscreteDynamicsWorld(dispatcher,broadphase,solver,collisionConfiguration);

	dynamicsWorld->setGravity(btVector3(0,-15080.0,0));

}

btDiscreteDynamicsWorld* PhysicsManager::GetDynamicsWorld()
{
	return dynamicsWorld;
}

void PhysicsManager::Update()
{
	int numManifolds = GetDynamicsWorld()->getDispatcher()->getNumManifolds();
	for (int i=0;i<numManifolds;i++)
	{
		btPersistentManifold* contactManifold =  GetDynamicsWorld()->getDispatcher()->getManifoldByIndexInternal(i);
		btCollisionObject* obA = static_cast<btCollisionObject*>(contactManifold->getBody0());
		btCollisionObject* obB = static_cast<btCollisionObject*>(contactManifold->getBody1());
		// if object a is a missile and object b is a ship act accordingly
		if( ((PhysicsContainer*)obA->getUserPointer())->type == 2 && ((PhysicsContainer*)obB->getUserPointer())->type == 1)
		{
			if(((PhysicsContainer*)obA->getUserPointer())->dead == false)
				player->hit_points--;
			((PhysicsContainer*)obA->getUserPointer())->dead = true;
			
		}
		// if object b is a missile and object a is a ship act accordingly
		if( ((PhysicsContainer*)obB->getUserPointer())->type == 2 && ((PhysicsContainer*)obA->getUserPointer())->type == 1)
		{
			if(((PhysicsContainer*)obB->getUserPointer())->dead == false)
				player->hit_points--;
			((PhysicsContainer*)obB->getUserPointer())->dead = true;
		}
		if( ((PhysicsContainer*)obB->getUserPointer())->type == 3 && ((PhysicsContainer*)obA->getUserPointer())->type == 1)
		{
			if(((PhysicsContainer*)obB->getUserPointer())->dead == false)
				player->cheese++;
			((PhysicsContainer*)obB->getUserPointer())->dead = true;
		}
		if( ((PhysicsContainer*)obB->getUserPointer())->type == 3 && ((PhysicsContainer*)obA->getUserPointer())->type == 1)
		{
			if(((PhysicsContainer*)obB->getUserPointer())->dead == false)
				player->cheese++;
			((PhysicsContainer*)obB->getUserPointer())->dead = true;
		}
	}

	GetDynamicsWorld()->stepSimulation(1/60.f,10);
	player->Update();
	player->fallRigidBody->setAngularFactor(btVector3(0,0,0));
	player->mouseJump.Update();
	player->fallRigidBody->applyCentralForce(btVector3(0,player->mouseJump.current_force,0));
	EntityManager * eMan = EntityManager::Instance();

	
	btScalar mMaxSpeed = btScalar(10.0f);
	btVector3 velocity = player->fallRigidBody->getLinearVelocity();
    btScalar speed = velocity.length();
    if(speed > mMaxSpeed) {
        velocity *= mMaxSpeed/speed;
        player->fallRigidBody->setLinearVelocity(velocity);
    }
	
	vector<Entity*> updatedEntities;
	// update the position and orientation of all entities based on the current properties of their rigid bodies
	for(int i = 0; i < eMan->entities.size(); i++)
	{
		eMan->entities[i]->Update();
		if(eMan->entities[i]->hit_points > 0)
		{
			updatedEntities.push_back(eMan->entities[i]);
			btTransform trans;
			eMan->entities[i]->GetRigidBody()->getMotionState()->getWorldTransform(trans);
			D3DXMatrixRotationQuaternion(&(eMan->entities[i]->matRotation), 
				&D3DXQUATERNION(trans.getRotation().getX(), trans.getRotation().getY(), trans.getRotation().getZ(), trans.getRotation().getW()));
			eMan->entities[i]->vRotation = D3DXVECTOR3(0,0,-1);
			D3DXVec3TransformCoord(&eMan->entities[i]->vRotation, &eMan->entities[i]->vRotation, &eMan->entities[i]->matRotation);
			D3DXVec3Normalize(&eMan->entities[i]->vRotation, &eMan->entities[i]->vRotation);
			eMan->entities[i]->SetPosition(D3DXVECTOR3(trans.getOrigin().getX(), trans.getOrigin().getY(), trans.getOrigin().getZ()));
		}
		// if the entity is dead clean its rigid body out
		else
		{
			eMan->entities[i]->dead = true;
			GetDynamicsWorld()->removeRigidBody(eMan->entities[i]->fallRigidBody);
		}
	}
	eMan->entities = updatedEntities;
	GraphicsManager* gMan = GraphicsManager::Instance();
	gMan->camHandler.mPos.x = player->fallRigidBody->getWorldTransform().getOrigin().getX();
	gMan->camHandler.mView.x = player->fallRigidBody->getWorldTransform().getOrigin().getX();
		//MiscUtils::CheckIntVal(j);
		//eMan->missiles[j]->Update();
	
		//D3DXMatrixRotationQuaternion(&(eMan->missiles[i]->matRotation), 
			//&D3DXQUATERNION(trans.getRotation().getX(), trans.getRotation().getY(), trans.getRotation().getZ(), trans.getRotation().getW()));
		//eMan->entities[i]->vRotation = D3DXVECTOR3(0,0,-1);
		//D3DXVec3TransformCoord(&eMan->missiles[i]->vRotation, &eMan->missiles[i]->vRotation, &eMan->missiles[i]->matRotation);
		//D3DXVec3Normalize(&eMan->missiles[i]->vRotation, &eMan->missiles[i]->vRotation);
}