#include "UIManager.hpp"

UIManager* UIManager::sInstance = NULL;

UIManager* UIManager::Instance()
{
	if(!sInstance)
		sInstance = new UIManager();
	return sInstance;
}

// if the user left clicks, this function is called
// it shoots a ray into the local space of every entity and checks to see if they interesect
// if they intersect that entity is made the new selected entity (if it is part of the player's faction
void UIManager::SelectAtCursor()
{
	POINT mousePos;
	GetCursorPos(&mousePos);

	// compute the ray in view space given the clicked screen point
	Ray ray = GraphicsManager::CalcPickingRay(mousePos.x, mousePos.y);

	// transform the ray to world space
	D3DXMATRIX view;
	GraphicsManager::Instance()->d3ddev->GetTransform(D3DTS_VIEW, &view);

	D3DXMATRIX viewInverse;
	D3DXMatrixInverse(&viewInverse,	0, &view);

	GraphicsManager::TransformRay(&ray, &(viewInverse));
	EntityManager* eMan = EntityManager::Instance();
	
	BOOL hasHit = false;
	for(int i = 0; i < eMan->entities.size(); i++)
	{
		float distanceToCollision = 0;
		Ray rayTemp;
		rayTemp = ray;

		D3DXMATRIX matTranslate;
		D3DXMatrixTranslation(&matTranslate, eMan->entities[i]->position.x, eMan->entities[i]->position.y, eMan->entities[i]->position.z);
		GraphicsManager::d3ddev->SetTransform(D3DTS_WORLD, &(matTranslate));

		// Use inverse of matrix
		D3DXMATRIX matInverse, matWorld;
		GraphicsManager::d3ddev->GetTransform(D3DTS_WORLD, &matWorld);
		D3DXMatrixInverse(&matInverse,NULL,&matWorld);

		// Transform ray origin and direction by inv matrix
		D3DXVECTOR3 rayObjOrigin,rayObjDirection;

		D3DXVec3TransformCoord(&rayObjOrigin,&ray._origin,&matInverse);
		D3DXVec3TransformNormal(&rayObjDirection,&ray._direction,&matInverse);
		D3DXVec3Normalize(&rayObjDirection,&rayObjDirection);
		GraphicsManager::rays.push_back(ray);
		BOOL hasHit;
		D3DXIntersect(eMan->entities[i]->mesh, &rayObjOrigin, &(rayObjDirection), &hasHit, NULL, NULL, NULL, &distanceToCollision, NULL, NULL);
		if(hasHit == true)
		{
			GraphicsManager::selectedEntity = eMan->entities[i];
		}
	}
	
}

// this function is called on right clicks
// if the ray hits an enemy ship the currently selected ship attacks it
// if the ray does not hit an enemy ship it will hit the x/z plane and order the currently selected ship to move there
void UIManager::ActionAtCursor()
{
	POINT mousePos;
	GetCursorPos(&mousePos);

	// compute the ray in view space given the clicked screen point
	Ray ray = GraphicsManager::CalcPickingRay(mousePos.x, mousePos.y);

	// transform the ray to world space
	D3DXMATRIX view;
	GraphicsManager::Instance()->d3ddev->GetTransform(D3DTS_VIEW, &view);

	D3DXMATRIX viewInverse;
	D3DXMatrixInverse(&viewInverse,	0, &view);

	GraphicsManager::TransformRay(&ray, &(viewInverse));
	if(GraphicsManager::selectedEntity != NULL && GraphicsManager::selectedEntity->faction == 0)
	{
		BOOL hasHit = false;
		EntityManager* eMan = EntityManager::Instance();
		for(int i = 0; i < eMan->entities.size(); i++)
		{
			float distanceToCollision = 0;
			Ray rayTemp;
			rayTemp = ray;

			D3DXMATRIX matTranslate;
			D3DXMatrixTranslation(&matTranslate, eMan->entities[i]->position.x, eMan->entities[i]->position.y, eMan->entities[i]->position.z);
			GraphicsManager::d3ddev->SetTransform(D3DTS_WORLD, &(matTranslate));

			// Use inverse of matrix
			D3DXMATRIX matInverse, matWorld;
			GraphicsManager::d3ddev->GetTransform(D3DTS_WORLD, &matWorld);
			D3DXMatrixInverse(&matInverse,NULL,&matWorld);

			// Transform ray origin and direction by inv matrix
			D3DXVECTOR3 rayObjOrigin,rayObjDirection;

			D3DXVec3TransformCoord(&rayObjOrigin,&ray._origin,&matInverse);
			D3DXVec3TransformNormal(&rayObjDirection,&ray._direction,&matInverse);
			D3DXVec3Normalize(&rayObjDirection,&rayObjDirection);
			GraphicsManager::rays.push_back(ray);
			
			D3DXIntersect(eMan->entities[i]->mesh, &rayObjOrigin, &(rayObjDirection), &hasHit, NULL, NULL, NULL, &distanceToCollision, NULL, NULL);
			if(hasHit == true && eMan->entities[i]->faction == 1)
			{
				GraphicsManager::selectedEntity->target = eMan->entities[i];
				break;
			}
		}
		if(hasHit == false)
		{
			D3DXVECTOR3 plane_point;
			D3DXPLANE plane(0.0f, 1.0f, 0.0f, 0.0f);
			ray._direction = (ray._direction*1000)+ray._origin;	
			D3DXPlaneIntersectLine(&plane_point, &plane, &ray._origin, &(ray._direction));
			GraphicsManager::selectedEntity->MoveTo(plane_point);
		}
	}
}