#include "WinHandler.hpp"

WinHandler::WinHandler(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow, WNDPROC mainWndProc)
{
    WNDCLASSEX wc;

    // clear out the memory we need at this pointer.
    ZeroMemory(&wc, sizeof(WNDCLASSEX));

    // fill in the struct with the needed information
    wc.cbSize = sizeof(WNDCLASSEX);
    wc.style = CS_HREDRAW | CS_VREDRAW;		//Lots of different style flags can be set here, we're telling it to redraw on horizontal or vertical moves.
	wc.lpfnWndProc = mainWndProc;			//Tells the window class the function to use when it gets a message from windows.
    wc.hInstance = hInstance;				//The handle windows has given us for this application.
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);	//Set one of the default mouse cursors
    wc.hbrBackground = (HBRUSH)COLOR_WINDOW;	//"brush" used to indicate the colour of the background.
    wc.lpszClassName = "WindowClass1";		//The name of the window class we are building.

    // register the window class
    RegisterClassEx(&wc);

    // create the window and use the result as the handle
    hWnd = CreateWindowEx(NULL,
                          "WindowClass1",    // name of the window class we have registered
                          "D3D Program",   // title of the window
                          WS_OVERLAPPEDWINDOW,    // window style, tons of flags for this, we can remove the minimize and maximize buttons, add scrollbars, etc.
                          0,    // x-position of the window
                          0,    // y-position of the window
                          1680,    // width of the window
                          1050,    // height of the window
                          NULL,    // we have no parent window, NULL
                          NULL,    // we aren't using menus, NULL
                          hInstance,    // application handle
                          NULL);    // used with multiple windows, NULL

    // display the window on the screen
    ShowWindow(hWnd,	//Handle to our window.
		nCmdShow);		//Commandline parameters from WinMain.
}

bool WinHandler::HandleWindowsMessages()
{
	// this struct holds Windows event messages
    MSG msg;

	while(PeekMessage(	//Returns false if there is no message in the queue to process.
		&msg,			//Pointer to where we wish to store the message.
		NULL,			//Handle to the window the message should come from. If it's null we accept messages to any of the windows created by this application.
		0,				//wMsgFilterMin
		0,				//wMsgFilterMax - These two paramaters can be used to filter which messages we want. Passing 0 means no filtering.
		PM_REMOVE))		//Whether we remove the event from the event queue or not.			
	{
		// translate messages into the right format
		TranslateMessage(&msg);

		// send the message to the WindowProc(ess) function
		DispatchMessage(&msg);
	}

	//If quit was posted, lets exit our main loop.
	if (msg.message == WM_QUIT)
	{
		return false;
	}

	return true;
}