// include the basic windows header file
#include <windows.h>

#include "WinHandler.hpp"
#include "NeheTimer.hpp"
#include "EntityManager.hpp"
#include "CCamera.hpp"
#include "Entity.hpp"
#include "GraphicsManager.hpp"
#include "UIManager.hpp"
#include "PhysicsManager.hpp"

//Windowproc prototype (handled in main, just so we have easier access to windows messages).
LRESULT CALLBACK WindowProc(HWND hWnd,
									 UINT message,
									 WPARAM wParam,
									 LPARAM lParam);

//Function prototypes
void CleanUp();
void CheckInput(float time);
void Draw();
void UpdateListenerPos();
void WriteToGraphicsBuffers();

//Handler variables
WinHandler *wHandler;
NeheTimer *neheTimer;
GraphicsManager * gMan;
PhysicsManager * pMan;
Entity *test;
Entity *test2;
bool up;
bool mouse_move = true;
Entity *test3;
MousePlayer * player;
FMODManager *fMan;



//Current keys down.
bool curKeys[256];
//
bool hasKeyPressBeenRegistered[256];
//Last time we checked keys
bool lastKeys[256];

bool	renderCollision = false;

// the entry point for any Windows program
int WINAPI WinMain(HINSTANCE hInstance,		//The handle(number) windows assigns to each application.						
                   HINSTANCE hPrevInstance,	//The handle(number) to the previous instance of this application (deprecated).
                   LPSTR lpCmdLine,			//This string is used if the exe was run from the command line with special parameters
                   int nCmdShow)			//Indicates how the window will appear (maximized, minimized, normal + other flags).
{
	float timeSinceLastCall;				//Variable to store our current delta time (change in time, in milliseconds).

	//Create our windows handler and pass a function pointer for windowproc.
	wHandler = new WinHandler(hInstance, hPrevInstance, lpCmdLine, nCmdShow, WindowProc);
	ShowCursor(true);

	GraphicsManager::Initialize(wHandler->hWnd);
	gMan = GraphicsManager::Instance();
	


	//Create our D3D handler, and initialize it.
	

	neheTimer = new NeheTimer();
	neheTimer->Initialize();
	
	PhysicsManager::Initialize();
	pMan = PhysicsManager::Instance();

	Entity::InitializeClass();
	
	EntityManager::Initialize();

	player = new MousePlayer(D3DXVECTOR3(1000,1000,0), "Vernin_Standing_02.X", "FILL_COLOUR.jpg");
	player->fallRigidBody->setLinearVelocity(btVector3(-1,0,0));
	player->fallRigidBody->setFriction(btScalar(0.01f));
	EntityManager::AddEntity(player);

	GraphicsManager::Instance()->player = player;

	PhysicsManager::Instance()->player = player;

	
	//Our main loop.
	while(true)
	{
		
		//If we get a quit message from our windows messages, break out of our main loop.
		if (!wHandler->HandleWindowsMessages())
			break;
		
		neheTimer->Update();
		
		timeSinceLastCall = neheTimer->Get_dt();
		CheckInput(timeSinceLastCall);
		pMan->Update();
		
		UpdateListenerPos();
		//if(mouse_move)
			//gMan->camHandler.Mouse_Move(1680, 1050);
		gMan->Draw(); 
		
	}

	//Clean up once we've quit.
	CleanUp();
	fMan->~FMODManager();
}

// update the position of the 3d listener to the camera position
void UpdateListenerPos()
{
	
	
}

void CheckInput(float time)
{
	EntityManager * eMan = EntityManager::Instance();
	//if (curKeys['W']) {gMan->camHandler.Move_Camera(CAMERASPEED * time);}

	//if (curKeys['A']) {gMan->camHandler.Strafe_Camera(-CAMERASPEED * time);}
	//if (curKeys['S']){gMan->camHandler.Move_Camera(-CAMERASPEED * time);}
	//if (curKeys['D']) {gMan->camHandler.Strafe_Camera(CAMERASPEED * time);}
	if (curKeys['A']) 
	{  
		player->fallRigidBody->applyCentralForce(btVector3(-90000,0,0));
	}
	if (curKeys['D']) 
	{  
		player->fallRigidBody->applyCentralForce(btVector3(90000,0,0));
		
	}
	//if (curKeys['C']) {gMan->camHandler.Fly_Camera(-CAMERASPEED * time);}
	//if (curKeys['F']) {gMan->camHandler.Fly_Camera(CAMERASPEED * time);}
	/*if (curKeys['Z'] && !hasKeyPressBeenRegistered['Z']) 
	{ 
		hasKeyPressBeenRegistered['Z'] = true;
		if(mouse_move)
			mouse_move = false;
		else
			mouse_move = true;
	}*/
	memcpy(lastKeys, curKeys, sizeof(curKeys));	
}

void CleanUp()
{
	delete wHandler;

}

// this is the main message handler for the program
//Basically the same parameters as the MSG struct.
LRESULT CALLBACK WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    // sort through and find what code to run for the message given
    switch(message)
    {
        // this message is read when the window is closed
        case WM_DESTROY:
            {
                // close the application entirely
                PostQuitMessage(0);
                return 0;
            } break;
		case WM_KEYDOWN:
			{	
				if (wParam == VK_ESCAPE)
					PostQuitMessage(0);
				else if(wParam == VK_UP)
					up = true;
				curKeys[wParam] = true;
				
			}break;
		case WM_KEYUP:
			{
				if(wParam == VK_UP)
					up = false;
				if(wParam == VK_SPACE)
					player->mouseJump.doJump();
				curKeys[wParam] = false;
				hasKeyPressBeenRegistered[wParam] = false;
			}break;
		case WM_LBUTTONDOWN:
			{
				UIManager::Instance()->SelectAtCursor();
			}break;
		case WM_RBUTTONDOWN:
			{
				//UIManager::Instance()->ActionAtCursor();
			}break;
			
		
    }

    // Handle any messages the switch statement didn't
    return DefWindowProc (hWnd, message, wParam, lParam);
}